#!/bin/sh
cat <<HELP > /dev/null

Поддержка нескольких языков:

    new-words -l lang URL

Например, для немецких текстов:

    new-words -l de URL

Или, предварительно создав соответствующую ссылку:

    de-words URL

HELP

WORK_DIR=~/.new-words/
TEMP1=`mktemp /tmp/news-words-XXXXXXXXXX`
TEMP2=`mktemp /tmp/news-words-XXXXXXXXXX`
export ORIGINAL_TEXT=`mktemp /tmp/news-words-XXXXXXXXXX-orig`
editor=${EDITOR:-vim}

LANGUAGE=en
my_name="`echo $0 | sed s@.*/@@ | sed s/-.*// `"
if echo "$1" | grep -q http://...wikipedia.org/wiki/
then
LANGUAGE="`echo $1 | sed s@http://@@ | sed s@.wikipedia.*@@`"
fi
[ "${my_name}" = "new" ] || LANGUAGE="$my_name"
if [ "$1" = "-l" ]
then
    LANGUAGE="$2"
    VOCABULARY="$LANGUAGE".txt
    shift 2
fi
VOCABULARY=${LANGUAGE}.txt
NOTES_FILE=notes-${LANGUAGE}.txt

get_words()
{
tr ' ' '\n' | sed 's/--/ /g' \
| sed "s/'/__APOSTROPHE__/g" \
| tr  '—·-' '-----' \
| tr '*\r,.:#@()+=—<>$;"?!|·[]^%&' '                           ' \
| tr ' ' '\n' \
| grep -x '[[:alpha:]'"'"'-]*' \
| tee "$1" \
| grep_v_english_perl \
| sort | uniq -c | awk '{if ($2!="") print;}' | sort -rn
}

add_stat()
{
    before="$1"
    after=${before}2
    cat > "$after"
    total="`wc -w $1 | awk '{print $1}'`"
    total_unknown="`cat $after|awk '{s=s+$1}END{print s}'`"
    total_known="`echo $total-$total_unknown|bc`"
    percentage="`echo '100*('$total-$total_unknown')'/$total | bc -l | sed 's/\\.\(.\).*/.\1/'`"
    echo "# $LANGUAGE, $percentage, <$total_known/$total>"
    cat "$after"
    rm $after
}

grep_v_english()
{
[ -e "$VOCABULARY" ] || touch "$VOCABULARY"
eval $(cat $VOCABULARY | tr -d "'" | xargs -n10 echo | tr ' ' '|' | sed 's/^/egrep -xv "RRRRRRR|/' | sed 's/$/"/' | tr '\n' '|')cat
}

grep_v_english_perl()
{
    PERL_SCRIPT_TEMP_NAME=`mktemp /tmp/perl-grep-v-english-XXXXXXXX`
    cat <<'PERL_SCRIPT' > $PERL_SCRIPT_TEMP_NAME
open(VOC, $ENV{VOCABULARY})
 or die "Can't open VOCABULARY";
while (<VOC>){
    chomp;
    #s/'//g;
    $voc{$_}="1";
}
while(<>) {
    chomp;
    if (not defined($voc{$_})) { print "$_\n"; }
}
PERL_SCRIPT
    [ -e "$VOCABULARY" ] || touch "$VOCABULARY"
    export VOCABULARY 
    perl $PERL_SCRIPT_TEMP_NAME
    rm $PERL_SCRIPT_TEMP_NAME
}

text_from_url()
{
lynx -dump "$1" | perl -p -e 's@http://[a-zA-Z&_.:/0-9%?=,#+()\[\]~-]*@@'
}

add_marks()
{
    PERL_SCRIPT_TEMP_NAME=`mktemp /tmp/perl-grep-v-english-XXXXXXXX`
    cat <<'PERL_SCRIPT' > $PERL_SCRIPT_TEMP_NAME
$file = $ARGV[0];
our $dict;
if (open(NOTES, $ENV{NOTES_FILE})) {
    while(<NOTES>) {
        chomp;
        s/^\s+//;
        my ($a,$b)=split /\s+/,$_,2;
        $dict{$a}=$b;
    }
}
if (open(F, $file)) {
    @lines=<F>;
    close(F);

    if (open(F, ">$file")) {
        for (@lines) {
            m/\s+\S+\s+(\S+)/;
            $name=$1;
            if (not /^#/ and defined($dict{$name})) {
                chomp;
                $mark=$dict{$name};
                $space=" "x(30-length($_));
                print F "$_$space$mark\n";
            }
            else {
                print F "$_";
            }
        }
        close(F);
    }
}
PERL_SCRIPT
    [ -e "$NOTES_FILE" ] || touch "$NOTES_FILE"
    export NOTES_FILE
    perl $PERL_SCRIPT_TEMP_NAME "$1"
    rm $PERL_SCRIPT_TEMP_NAME
}

remove_marks()
{
    PERL_SCRIPT_TEMP_NAME=`mktemp /tmp/perl-grep-v-english-XXXXXXXX`
    cat <<'PERL_SCRIPT' > $PERL_SCRIPT_TEMP_NAME
$file = $ARGV[0];
our %dict;
if (open(F, $file)) {
    @lines=<F>;
    close(F);

    if (open(F, ">$file")) {
        for (@lines) {
            chomp;
            if (not /^#/ and m/(\s+)(\S+)(\s+)(\S+)(\s+)(.*)/) {
                my $name=$4;
                my $comment=$6;
                $dict{$name}=$comment;
                print F "$1$2$3$4\n";
            }
            else {
                print F "$_\n";
            }
        }
    }
}
if (open(NOTES, $ENV{NOTES_FILE})) {
    @lines=<NOTES>;
    close(NOTES);

    if (open(NOTES, ">".$ENV{NOTES_FILE})) {
        for (@lines) {
            chomp;
            s/^\s+//;
            my ($a,$b)=split /\s+/,$_,2;
            if (not defined($dict{$a}) || ($dict{$a} eq $b)) {
                print NOTES "$_\n";
                if (defined($dict{$a})) { unset($dict{$a}); }
            }
        }
        for (keys %dict) {
            $mark=$dict{$_};
            $space=" "x(30-length($_));
            print NOTES "$_$space$mark\n";
        }
    }
}
PERL_SCRIPT
    [ -e "$NOTES_FILE" ] || touch "$NOTES_FILE"
    export NOTES_FILE
    perl $PERL_SCRIPT_TEMP_NAME "$1"
    rm $PERL_SCRIPT_TEMP_NAME
}

mkdir -p $WORK_DIR
oldpwd="$PWD"
cd $WORK_DIR
if echo "$1" | grep -q http: 
then 
    text_from_url "$1" | tee $ORIGINAL_TEXT \
    | get_words ${TEMP1}-full | add_stat ${TEMP1}-full| tee "$TEMP1" > "$TEMP2"
elif [ "$#" != 0 ]
then
    cat "$oldpwd/$1" | tee $ORIGINAL_TEXT \
    | get_words ${TEMP1}-full | add_stat ${TEMP1}-full | tee "$TEMP1" > "$TEMP2"
else 
    tee $ORIGINAL_TEXT \
    | get_words ${TEMP1}-full | add_stat ${TEMP1}-full | tee "$TEMP1" > "$TEMP2"
fi

add_marks "$TEMP2"
if [ "$editor" = vim ]
then
    vim -c 'set keywordprg='"$LANGUAGE" -c 'set iskeyword=@,48-57,/,.,-,_,+,,,#,$,%,~,=' "$TEMP2" < /dev/tty > /dev/tty
else
    echo 2
    $editor "$TEMP2"
fi
remove_marks "$TEMP2"

diff "$TEMP1" "$TEMP2" | awk '{print $3}' | sort -u >> "$VOCABULARY"
rm -f "$TEMP1" "$TEMP2" "${TEMP1}-full" "$ORIGINAL_TEXT"
